//+------------------------------------------------------------------+
//|                                              ControlsDialog3.mqh |
//|                                    Copyright (c) 2019, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+

#include <Controls\Dialog.mqh>
#include <Controls\Button.mqh>
#include <Controls\Edit.mqh>
#include <Controls\DatePicker.mqh>
#include <Controls\ListView.mqh>
#include <Controls\ComboBox.mqh>
#include <Controls\SpinEdit.mqh>
#include <Controls\RadioGroup.mqh>
#include <Controls\CheckGroup.mqh>
#include <Layouts/Box.mqh>
#include <Layouts/SpinEditResizable.mqh>
#include <Layouts/ComboBoxResizable.mqh>
#include <Layouts/MaximizableAppDialog.mqh>

//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
// buttons
#define BUTTON_WIDTH (100)
#define BUTTON_HEIGHT (20)
// editbox
#define EDIT_HEIGHT (20)
// groups
#define GROUP_WIDTH (150)
#define LIST_HEIGHT (179)
#define RADIO_HEIGHT (56)
#define CHECK_HEIGHT (93)

//+-----------------------------------------------------------------------+
//| CControlsDialog                                                       |
//| Main dialog window with controls                                      |
//+-----------------------------------------------------------------------+
class CControlsDialog: public MaximizableAppDialog // CAppDialog
{
  private:
    CBox m_main;
    CBox m_edit_row;
    CEdit m_edit;
    CBox m_button_row;
    CButton m_button1;
    CButton m_button2;
    CButton m_button3;
    CBox m_spin_date_row;
    //CSpinEdit m_spin_edit;
    SpinEditResizable m_spin_edit;
    CDatePicker m_date;
    CBox m_lists_row;
    CBox m_lists_column1;
    //CComboBox m_combo_box;
    ComboBoxResizable m_combo_box;
    CRadioGroup m_radio_group;
    CCheckGroup m_check_group;
    CBox m_lists_column2;
    CListView m_list_view;

  public:
    CControlsDialog(void);
    ~CControlsDialog(void);

    virtual bool Create(const long chart, const string name, const int subwin, const int x1, const int y1, const int x2, const int y2);
    // general event handler
    virtual bool OnEvent(const int id, const long &lparam, const double &dparam, const string &sparam);

  protected:
    // controls
    CWnd *CreateEdit(void);
    CWnd *CreateButton1(void);
    CWnd *CreateButton2(void);
    CWnd *CreateButton3(void);
    CWnd *CreateSpinEdit(void);
    CWnd *CreateDate(void);
    CWnd *CreateListView(void);
    CWnd *CreateComboBox(void);
    CWnd *CreateRadioGroup(void);
    CWnd *CreateCheckGroup(void);

    // specific handlers
    void OnClickButton1(void);
    void OnClickButton2(void);
    void OnClickButton3(void);
    void OnChangeSpinEdit(void);
    void OnChangeDate(void);
    void OnChangeListView(void);
    void OnChangeComboBox(void);
    void OnChangeRadioGroup(void);
    void OnChangeCheckGroup(void);

    // containers
    virtual CWnd *CreateMain(const long chart, const string name, const int subwin);
    virtual CWnd *CreateEditRow(const long chart, const string name, const int subwin);
    virtual CWnd *CreateButtonRow(const long chart, const string name, const int subwin);
    virtual CWnd *CreateSpinDateRow(const long chart, const string name, const int subwin);
    virtual CWnd *CreateListsRow(const long chart, const string name, const int subwin);
    virtual CWnd *CreateListsColumn1(const long chart, const string name, const int subwin);
    virtual CWnd *CreateListsColumn2(const long chart, const string name, const int subwin);
    
    virtual void SelfAdjustment(const bool restore = false) override;
};

//+------------------------------------------------------------------+
//| Event handling                                                   |
//+------------------------------------------------------------------+
EVENT_MAP_BEGIN(CControlsDialog)
  ON_EVENT(ON_CLICK, m_button1, OnClickButton1)
  ON_EVENT(ON_CLICK, m_button2, OnClickButton2)
  ON_EVENT(ON_CLICK, m_button3, OnClickButton3)
  ON_EVENT(ON_CHANGE, m_spin_edit, OnChangeSpinEdit)
  ON_EVENT(ON_CHANGE, m_date, OnChangeDate)
  ON_EVENT(ON_CHANGE, m_list_view, OnChangeListView)
  ON_EVENT(ON_CHANGE, m_combo_box, OnChangeComboBox)
  ON_EVENT(ON_CHANGE, m_radio_group, OnChangeRadioGroup)
  ON_EVENT(ON_CHANGE, m_check_group, OnChangeCheckGroup)
EVENT_MAP_END(MaximizableAppDialog)

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CControlsDialog::CControlsDialog(void)
{
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CControlsDialog::~CControlsDialog(void)
{
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CControlsDialog::Create(const long chart, const string name, const int subwin, const int x1, const int y1, const int x2, const int y2)
{
    if(MaximizableAppDialog::Create(chart, name, subwin, x1, y1, x2, y2)
    && Add(CreateMain(chart, name, subwin)))
    {
        return true;
    }
    return false;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateMain(const long chart, const string name, const int subwin)
{
    m_main.LayoutStyle(LAYOUT_STYLE_VERTICAL);
    if(m_main.Create(chart, name + "main", subwin, 0, 0, ClientAreaWidth(), ClientAreaHeight())
    && m_main.Add(CreateEditRow(chart, name, subwin))
    && m_main.Add(CreateButtonRow(chart, name, subwin))
    && m_main.Add(CreateSpinDateRow(chart, name, subwin))
    && m_main.Add(CreateListsRow(chart, name, subwin))
    && m_main.Pack())
    {
        SelfAdjustment();
        return &m_main;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateEditRow(const long chart, const string name, const int subwin)
{
    if(m_edit_row.Create(chart, name + "editrow", subwin, 0, 0, ClientAreaWidth(), EDIT_HEIGHT * 1.5)
    && m_edit_row.Add(CreateEdit()))
    {
        m_edit_row.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);//|WND_ALIGN_TOP|WND_ALIGN_BOTTOM
        return &m_edit_row;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateButtonRow(const long chart, const string name, const int subwin)
{
    if(m_button_row.Create(chart, name + "buttonrow", subwin, 0, 0, ClientAreaWidth(), BUTTON_HEIGHT * 1.5)
    && m_button_row.Add(CreateButton1())
    && m_button_row.Add(CreateButton2())
    && m_button_row.Add(CreateButton3()))
    {
      m_button_row.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);
      return &m_button_row;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateSpinDateRow(const long chart, const string name, const int subwin)
{
    if(m_spin_date_row.Create(chart, name + "spindaterow", subwin, 0, 0, ClientAreaWidth(), BUTTON_HEIGHT * 1.5)
    && m_spin_date_row.Add(CreateSpinEdit())
    && m_spin_date_row.Add(CreateDate()))
    {
        m_spin_date_row.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);
        return &m_spin_date_row;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateListsRow(const long chart, const string name, const int subwin)
{
    m_lists_row.PaddingLeft(0);
    m_lists_row.PaddingRight(0);
    const int h = m_edit_row.Height() + m_button_row.Height() + m_spin_date_row.Height();
    m_lists_row.Alignment(WND_ALIGN_CLIENT, 2, h, 2, 4);

    if(m_lists_row.Create(chart, name + "listsrow", subwin, 0, 0, ClientAreaWidth(), LIST_HEIGHT)
    && m_lists_row.Add(CreateListsColumn1(chart, name, subwin))
    && m_lists_row.Add(CreateListsColumn2(chart, name, subwin)))
    {
        return &m_lists_row;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateListsColumn1(const long chart, const string name, const int subwin)
{
    m_lists_column1.Padding(0);
    m_lists_column1.LayoutStyle(LAYOUT_STYLE_VERTICAL);
    m_lists_column1.VerticalAlign(VERTICAL_ALIGN_CENTER_NOSIDES);

    if(m_lists_column1.Create(chart, name + "listscolumn1", subwin, 0, 0, GROUP_WIDTH, LIST_HEIGHT)
    && m_lists_column1.Add(CreateComboBox())
    && m_lists_column1.Add(CreateRadioGroup())
    && m_lists_column1.Add(CreateCheckGroup()))
    {
        return &m_lists_column1;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateListsColumn2(const long chart, const string name, const int subwin)
{
    m_lists_column2.Padding(0);
    m_lists_column2.LayoutStyle(LAYOUT_STYLE_VERTICAL);
    m_lists_column2.VerticalAlign(VERTICAL_ALIGN_CENTER);

    if(m_lists_column2.Create(chart, name + "listscolumn2", subwin, 0, 0, GROUP_WIDTH, LIST_HEIGHT)
    && m_lists_column2.Add(CreateListView()))
    {
        return &m_lists_column2;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateEdit(void)
{
    if(m_edit.Create(m_chart_id, m_name + "Edit", m_subwin, 0, 0, ClientAreaWidth(), EDIT_HEIGHT)
    && m_edit.ReadOnly(true))
    {
      m_edit.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 4, 0, 4, 0);//|WND_ALIGN_TOP|WND_ALIGN_BOTTOM
      return &m_edit;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateButton1(void)
{
    if(m_button1.Create(m_chart_id, m_name + "Button1", m_subwin, 0, 0, BUTTON_WIDTH, BUTTON_HEIGHT)
    && m_button1.Text("Button1"))
    {
      return &m_button1;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateButton2(void)
{
    if(m_button2.Create(m_chart_id, m_name + "Button2", m_subwin, 0, 0, BUTTON_WIDTH, BUTTON_HEIGHT)
    && m_button2.Text("Button2"))
    {
        return &m_button2;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateButton3(void)
{
    if(m_button3.Create(m_chart_id, m_name + "Button3", m_subwin, 0, 0, BUTTON_WIDTH, BUTTON_HEIGHT)
    && m_button3.Text("Locked"))
    {
        m_button3.Locking(true);
        return &m_button3;
    }    
    
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateSpinEdit(void)
{
    if(m_spin_edit.Create(m_chart_id, m_name + "SpinEdit", m_subwin, 0, 0, GROUP_WIDTH, EDIT_HEIGHT))
    {
        m_spin_edit.MinValue(10);
        m_spin_edit.MaxValue(1000);
        m_spin_edit.Value(100);
        return &m_spin_edit;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateDate(void)
{
    if(m_date.Create(m_chart_id, m_name + "Date", m_subwin, 0, 0, GROUP_WIDTH, EDIT_HEIGHT))
    {
        m_date.Value(TimeCurrent());
        return &m_date;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateListView(void)
{
    if(m_list_view.Create(m_chart_id, m_name + "ListView", m_subwin, 0, 0, GROUP_WIDTH, LIST_HEIGHT))
    {
      m_list_view.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);
      for(int i = 0; i < 16; i++)
        if(!m_list_view.AddItem("Item " + IntegerToString(i)))
            return NULL;
      return &m_list_view;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateComboBox(void)
{
    if(m_combo_box.Create(m_chart_id, m_name + "ComboBox", m_subwin, 0, 0, GROUP_WIDTH, EDIT_HEIGHT))
    {
      m_combo_box.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);
      for(int i = 0; i < 16; i++)
        if(!m_combo_box.ItemAdd("Item " + IntegerToString(i)))
            return NULL;
      return &m_combo_box;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateRadioGroup(void)
{
    if(m_radio_group.Create(m_chart_id, m_name + "RadioGroup", m_subwin, 0, 0, GROUP_WIDTH, RADIO_HEIGHT))
    {
      m_radio_group.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);
      for(int i = 0; i < 3; i++)
        if(!m_radio_group.AddItem("Item " + IntegerToString(i), 1 << i))
            return NULL;
      return &m_radio_group;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CWnd *CControlsDialog::CreateCheckGroup(void)
{
    if(m_check_group.Create(m_chart_id, m_name + "CheckGroup", m_subwin, 0, 0, GROUP_WIDTH, CHECK_HEIGHT))
    {
      m_check_group.Alignment(WND_ALIGN_LEFT|WND_ALIGN_RIGHT, 2, 0, 2, 0);
      for(int i = 0; i < 5; i++)
        if(!m_check_group.AddItem("Item " + IntegerToString(i), 1 << i))
            return NULL;
      return &m_check_group;
    }
    return NULL;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnClickButton1(void)
{
    m_edit.Text(__FUNCTION__);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnClickButton2(void)
{
    m_edit.Text(__FUNCTION__ + " " + (string)m_button2.StateFlags());
    m_button2.StateFlagsReset(7);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnClickButton3(void)
{
    if(m_button3.Pressed())
        m_edit.Text(__FUNCTION__ + "On");
    else
        m_edit.Text(__FUNCTION__ + "Off");
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnChangeSpinEdit()
{
    m_edit.Text(__FUNCTION__ + " : Value=" + IntegerToString(m_spin_edit.Value()));
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnChangeDate(void)
{
    m_edit.Text(__FUNCTION__ + " \"" + TimeToString(m_date.Value(), TIME_DATE) + "\"");
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnChangeListView(void)
{
    m_edit.Text(__FUNCTION__ + " \"" + m_list_view.Select() + "\"");
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnChangeComboBox(void)
{
    m_edit.Text(__FUNCTION__ + " \"" + m_combo_box.Select() + "\"");
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnChangeRadioGroup(void)
{
    m_edit.Text(__FUNCTION__ + " : Value=" + IntegerToString(m_radio_group.Value()));
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CControlsDialog::OnChangeCheckGroup(void)
{
    m_edit.Text(__FUNCTION__ + " : Value=" + IntegerToString(m_check_group.Value()));
}

//+------------------------------------------------------------------+

void CControlsDialog::SelfAdjustment(const bool restore = false)
{
  CSize min = m_main.GetMinSize();
  CSize size;
  size.cx = ClientAreaWidth();
  size.cy = ClientAreaHeight();
  if(restore)
  {
    if(min.cx > size.cx) size.cx = min.cx;
    if(min.cy > size.cy) size.cy = min.cy;
  }
  m_main.Size(size);
  int w = (m_button_row.Width() - 2 * 2 * 2 * 3) / 3;
  m_button1.Width(w);
  m_button2.Width(w);
  m_button3.Width(w);
  m_edit.Width(w);
  m_spin_edit.Width(w);
  w = m_lists_row.Width() / 2;
  m_lists_column1.Width(w);
  m_lists_column2.Width(w);
  m_combo_box.Width(w);
  m_main.Pack();
}
