//+------------------------------------------------------------------+
//|                                                     PlotDemo.mqh |
//|                                    Copyright (c) 2019, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//+------------------------------------------------------------------+

#include <Controls\Dialog.mqh>
#include <PairPlot/Plot.mqh>
#include <Layouts/MaximizableAppDialog.mqh>

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CPlotDemo: public MaximizableAppDialog // CAppDialog
{
  private:
    CPlot m_plot;

  public:
    CPlotDemo();
    ~CPlotDemo();

    bool Create(const long chart, const string name, const int subwin, const int x1, const int y1, const int x2, const int y2, const ENUM_CURVE_TYPE curveType = CURVE_POINTS);
    virtual bool OnEvent(const int id, const long &lparam, const double &dparam, const string &sparam);
    bool Refresh(void);

    virtual void SelfAdjustment(const bool minimized = false) override
    {
      if(!minimized)
      {
        m_plot.Size(ClientAreaWidth(), ClientAreaHeight());
        m_plot.Resize(0, 0, ClientAreaWidth(), ClientAreaHeight());
      }
      m_plot.Refresh();
    }
};

EVENT_MAP_BEGIN(CPlotDemo)
EVENT_MAP_END(MaximizableAppDialog)

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CPlotDemo::CPlotDemo()
{
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CPlotDemo::~CPlotDemo()
{
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CPlotDemo::Create(const long chart, const string name, const int subwin, const int x1, const int y1, const int x2, const int y2, const ENUM_CURVE_TYPE curveType = CURVE_POINTS)
{
    const int maxw = (int)ChartGetInteger(0, CHART_WIDTH_IN_PIXELS);
    const int maxh = (int)ChartGetInteger(0, CHART_HEIGHT_IN_PIXELS);
    int _x1 = x1;
    int _y1 = y1;
    int _x2 = x2;
    int _y2 = y2;
    if(x2 - x1 > maxw || x2 > maxw)
    {
      _x1 = 0;
      _x2 = _x1 + maxw - 0;
    }
    if(y2 - y1 > maxh || y2 > maxh)
    {
      _y1 = 0;
      _y2 = _y1 + maxh - 1;
    }
    
    if(!MaximizableAppDialog::Create(chart, name, subwin, _x1, _y1, _x2, _y2))
        return false;
    if(!m_plot.Create(m_chart_id, m_name + "Plot", m_subwin, 0, 0, ClientAreaWidth(), ClientAreaHeight(), curveType))
        return false;
    if(!Add(m_plot))
        return false;
    double x[] = {-10, -4, -1, 2, 3, 4, 5, 6, 7, 8};
    double y[] = {-5, 4, -10, 23, 17, 18, -9, 13, 17, 4};
    m_plot.CurveAdd(x, y, "Example 1");
    m_plot.CurveAdd(y, x, "Example 2");
    return true;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CPlotDemo::Refresh(void)
{
    return m_plot.Refresh();
}
//+------------------------------------------------------------------+
