//+------------------------------------------------------------------+
//|                                               SlidingPuzzle3.mqh |
//|                                    Copyright (c) 2019, Marketeer |
//|                          https://www.mql5.com/en/users/marketeer |
//|                     Based on original Enrico Lambino application |
//|                             https://www.mql5.com/en/users/iceron |
//+------------------------------------------------------------------+

#include <Layouts\GridTk.mqh>
#include <Layouts\MaximizableAppDialog.mqh>
#include <Controls\Edit.mqh>
#include <Controls\Button.mqh>

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CSlidingPuzzleDialog: public MaximizableAppDialog // CAppDialog
{
 protected:
  bool m_solved;
  int m_difficulty;
  CGridTk m_main;
  CButton m_buttons[16];
  CButton m_button_new;
  CEdit m_label;
  CButton *m_empty_cell;
  
 public:
  CSlidingPuzzleDialog();
  ~CSlidingPuzzleDialog();
  virtual bool Create(const long chart, const string name, const int subwin, const int x1, const int y1, const int x2, const int y2);
  virtual bool OnEvent(const int id, const long &lparam, const double &dparam, const string &sparam);
  void Difficulty(int d)
  {
    m_difficulty = d;
  }

 protected:
  virtual bool CreateMain(const long chart, const string name, const int subwin);
  virtual bool CreateButton(const int button_id, const long chart, const string name, const int subwin);
  virtual bool CreateButtonNew(const long chart, const string name, const int subwin);
  virtual bool CreateLabel(const long chart, const string name, const int subwin);
  virtual bool IsMovable(CButton *button);
  virtual bool HasNorth(CButton *button, int id, bool shuffle = false);
  virtual bool HasSouth(CButton *button, int id, bool shuffle = false);
  virtual bool HasEast(CButton *button, int id, bool shuffle = false);
  virtual bool HasWest(CButton *button, int id, bool shuffle = false);
  virtual void Swap(CButton *source);
  bool OnClickButton(const int i);
  void OnClickButtonNew();
  bool Check();
  void Shuffle();
  
  virtual void SelfAdjustment(const bool restore = false) override;

};

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
EVENT_MAP_BEGIN(CSlidingPuzzleDialog)
  ON_INDEXED_EVENT(ON_CLICK, m_buttons, OnClickButton)
  ON_EVENT(ON_CLICK, m_button_new, OnClickButtonNew)
EVENT_MAP_END(MaximizableAppDialog)
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CSlidingPuzzleDialog::CSlidingPuzzleDialog():
    m_solved(false),
    m_difficulty(30)
{
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CSlidingPuzzleDialog::~CSlidingPuzzleDialog()
{
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::Create(const long chart, const string name, const int subwin, const int x1, const int y1, const int x2, const int y2)
{
  if(!MaximizableAppDialog::Create(chart, name, subwin, x1, y1, x2, y2))
    return (false);

  if(!CreateMain(chart, name, subwin))
    return (false);
  for(int i = 1; i <= 16; i++)
  {
    if(!CreateButton(i, chart, "block", subwin))
      return (false);
  }
  m_empty_cell = &m_buttons[15];
  if(!CreateButtonNew(chart, name, subwin))
    return (false);
  if(!CreateLabel(chart, name, subwin))
    return (false);
  if(!m_main.Pack())
    return (false);
  if(!Add(m_main))
    return (false);

  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::CreateMain(const long chart, const string name, const int subwin)
{
  if(!m_main.Create(chart, name + "main", subwin, 0, 0, ClientAreaWidth(), ClientAreaHeight()))
    return (false);
  m_main.Init(5, 4, 2, 2);
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::CreateButtonNew(const long chart, const string name, const int subwin)
{
  if(!m_button_new.Create(chart, name + "buttonnew", m_subwin, 0, 0, 101, 101))
    return (false);
  m_button_new.Text("New");
  m_button_new.ColorBackground(clrYellow);
  if(!m_main.Grid(GetPointer(m_button_new), 4, 0, 1, 2))
    return (false);
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::CreateLabel(const long chart, const string name, const int subwin)
{
  if(!m_label.Create(chart, name + "labelnew", m_subwin, 0, 0, 102, 102))
    return (false);
  m_label.Text("click new");
  m_label.ReadOnly(true);
  m_label.TextAlign(ALIGN_CENTER);
  if(!m_main.Grid(GetPointer(m_label), 4, 2, 1, 2))
    return (false);
    
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::CreateButton(const int button_id, const long chart, const string name, const int subwin)
{
  CButton *button = &m_buttons[button_id - 1];
  if(!button.Create(chart, name + IntegerToString(button_id), subwin, 0, 0, 100, 100))
    return (false);
  if(button_id < 16)
  {
    if(!button.Text(IntegerToString(button_id)))
      return (false);
  }
  button.Id(button_id);
  button.ColorBackground(clrCyan);
  if(!m_main.Add(button))
    return (false);
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::IsMovable(CButton *button)
{
  int id = (int)(button.Id() - m_buttons[0].Id() + 1);
  if(button.Text() == "")
    return (false);
  if(HasNorth(button, id) || HasSouth(button, id) || HasEast(button, id) || HasWest(button, id))
    return (true);
  return (false);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::HasNorth(CButton *button, int id, bool shuffle = false)
{
  if(id == 1 || id == 2 || id == 3 || id == 4)
    return (false);
  CButton *button_adj = m_main.Control(id - 4);
  if(!CheckPointer(button_adj))
    return (false);
  if(!shuffle)
  {
    if(button_adj.Text() != "")
      return (false);
  }
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::HasSouth(CButton *button, int id, bool shuffle = false)
{
  if(id == 13 || id == 14 || id == 15 || id == 16)
    return (false);
  CButton *button_adj = m_main.Control(id + 4);
  if(!CheckPointer(button_adj))
    return (false);
  if(!shuffle)
  {
    if(button_adj.Text() != "")
      return (false);
  }
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::HasEast(CButton *button, int id, bool shuffle = false)
{
  if(id == 4 || id == 8 || id == 12 || id == 16)
    return (false);
  CButton *button_adj = m_main.Control(id + 1);
  if(!CheckPointer(button_adj))
    return (false);
  if(!shuffle)
  {
    if(button_adj.Text() != "")
      return (false);
  }
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::HasWest(CButton *button, int id, bool shuffle = false)
{
  if(id == 1 || id == 5 || id == 9 || id == 13)
    return (false);
  CButton *button_adj = m_main.Control(id - 1);
  if(!CheckPointer(button_adj))
    return (false);
  if(!shuffle)
  {
    if(button_adj.Text() != "")
      return (false);
  }
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CSlidingPuzzleDialog::Swap(CButton *source)
{
  string source_text = source.Text();
  string target_text = m_empty_cell.Text();
  source.Text(target_text);
  m_empty_cell.Text(source_text);
  m_empty_cell = source;
  m_empty_cell.Text("");
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::OnClickButton(const int i)
{
  CButton *button = &m_buttons[i];
  if(IsMovable(button) && !m_solved)
  {
    Swap(button);
    Check();
  }
  return true;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CSlidingPuzzleDialog::OnClickButtonNew(void)
{
  Shuffle();
  m_label.Text("not solved");
  m_solved = false;
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CSlidingPuzzleDialog::Check(void)
{
  int cnt = 1;
  for(int i = 0; i < m_main.ControlsTotal(); i++)
  {
    CWnd *control = m_main.Control(i);
    if(StringFind(control.Name(), "block") >= 0)
    {
      CButton *button = control;
      if(CheckPointer(button))
      {
        if(button.Text() != IntegerToString(cnt) && cnt < 16)
        {
          m_label.Text("not solved");
          return (false);
        }
      }
      cnt++;
    }
  }
  m_label.Text("solved");
  m_solved = true;
  return (true);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSlidingPuzzleDialog::Shuffle(void)
{
  int cnt = 1;
  for(int i = 1; i < m_main.ControlsTotal(); i++)
  {
    CWnd *control = m_main.Control(i);
    if(StringFind(control.Name(), "block") >= 0)
    {
      CButton *button = control;
      if(cnt < 16)
        button.Text((string)cnt);
      if(button.Id() - m_buttons[0].Id() + 1 == 16)
        m_empty_cell = button;
      cnt++;
    }
  }
  MathSrand((int)TimeLocal());
  CButton *target = NULL;
  for(int i = 0; i < m_difficulty; i++)
  {
    int empty_cell_id = (int)(m_empty_cell.Id() - m_buttons[0].Id() + 1);
    int random = MathRand() % 4 + 1;
    if(random == 1 && HasNorth(m_empty_cell, empty_cell_id, true))
      target = m_main.Control(empty_cell_id - 4);
    else if(random == 2 && HasEast(m_empty_cell, empty_cell_id, true))
      target = m_main.Control(empty_cell_id + 1);
    else if(random == 3 && HasSouth(m_empty_cell, empty_cell_id, true))
      target = m_main.Control(empty_cell_id + 4);
    else if(random == 4 && HasWest(m_empty_cell, empty_cell_id, true))
      target = m_main.Control(empty_cell_id - 1);
    if(CheckPointer(target))
      Swap(target);
  }
}
//+------------------------------------------------------------------+

void CSlidingPuzzleDialog::SelfAdjustment(const bool restore = false)
{
  CSize size;
  size.cx = ClientAreaWidth();
  size.cy = ClientAreaHeight();
  m_main.Size(size);
  m_main.Pack();
}
